/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt (hewitt@netscape.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== toolbarbutton.css =====================================================
  == Styles used by the XUL button element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: toolbarbutton ::::: */

toolbarbutton {
  -moz-appearance: toolbarbutton;
  -moz-box-align: center;
  -moz-box-pack: center;
  margin: 0;
  border: 1px solid transparent;
  padding: 3px;
  background-color: transparent;
  color: -moz-DialogText;
}

.toolbarbutton-icon {
  -moz-margin-end: 2px;
}

.toolbarbutton-text {
  margin: 0 !important;
  text-align: center;
}

toolbarbutton.tabbable {
  -moz-user-focus: normal !important;
}

toolbarbutton:focus 
{
  /* -moz-appearance looks redundant here but is necessary.
      Without it, the outline won't appear. */
  -moz-appearance: toolbarbutton;
  outline: 1px dotted -moz-DialogText;
  -moz-outline-offset: -2px;
}

toolbarbutton:hover,
toolbarbutton[buttonover="true"] {
  border-color: ThreeDHighlight ThreeDShadow ThreeDShadow ThreeDHighlight;
  color: -moz-buttonhovertext;
  background-color: -moz-buttonhoverface;
}

toolbarbutton:hover:active,
toolbarbutton[open="true"] {
  border-color: ThreeDShadow ThreeDHighlight ThreeDHighlight ThreeDShadow;
  padding-top: 4px;
  padding-bottom: 2px;
  -moz-padding-start: 4px;
  -moz-padding-end: 2px;
  color: ButtonText;
}

toolbarbutton[disabled="true"],
toolbarbutton[disabled="true"]:hover,
toolbarbutton[disabled="true"]:hover:active,
toolbarbutton[disabled="true"][open="true"] {
  border-color: transparent;
  padding: 3px;
  color: GrayText;
}

/* ..... checked state ..... */

toolbarbutton[checked="true"] {
  border-color: ThreeDShadow ThreeDHighlight ThreeDHighlight ThreeDShadow !important;
  padding-top: 4px !important;
  padding-bottom: 2px !important;
  -moz-padding-start: 4px !important;
  -moz-padding-end: 2px !important;
  background-image: url("chrome://global/skin/toolbar/Lighten.png");
  color: ButtonText !important;
}

/* ::::: toolbarbutton menu ::::: */

.toolbarbutton-menu-dropmarker {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

.toolbarbutton-menu-dropmarker[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-dis.gif");
}

/* ::::: toolbarbutton menu-button ::::: */

toolbarbutton[type="menu-button"] {
  -moz-box-align: stretch;
  -moz-appearance: dualbutton;
  -moz-box-orient: horizontal !important;
}

toolbarbutton[type="menu-button"],
toolbarbutton[type="menu-button"]:hover,
toolbarbutton[type="menu-button"]:hover:active,
toolbarbutton[type="menu-button"][open="true"],
toolbarbutton[type="menu-button"][disabled="true"],
toolbarbutton[type="menu-button"][disabled="true"]:hover,
toolbarbutton[type="menu-button"][disabled="true"]:hover:active {
  border-style: none;
  padding: 0 !important;
}

.toolbarbutton-menubutton-button {
  -moz-box-align: center;
  -moz-box-pack: center;
  -moz-box-orient: vertical;
}

/* .......... dropmarker .......... */

.toolbarbutton-menubutton-dropmarker {
  -moz-box-align: center;
  padding: 3px;
  list-style-image: url("chrome://global/skin/arrow/arrow-dn.gif");
  -moz-image-region: auto; /* cut off inheritance */
  -moz-appearance: dualbutton-dropdown;
}

.toolbarbutton-menubutton-dropmarker[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-dis.gif") !important;
}

toolbarbutton[type="menu-button"][open="true"] > .toolbarbutton-menubutton-dropmarker,
toolbarbutton[type="menu-button"][buttondown="true"] > .toolbarbutton-menubutton-dropmarker {
}
